/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/

#include "assert.h"

#ifndef INC_B4SHANDLER_H
#include "b4shandler.h"
#define INC_B4SHANDLER_H
#endif

#ifndef INC_DOMOGLOBALS_H
#include "domoglobals.h"
#define INC_DOMOGLOBALS_H
#endif

#ifndef INC_FILEIO_H
#include "fileio.h"
#define INC_FILEIO_H
#endif

#ifndef INC_STRINGMANIP_H
#include "stringmanip.h"
#define INC_STRINGMANIP_H
#endif

#include <iostream>

using std::cout;
using B4sHandlerSpace::B4sHandler;
using TrackSpace::Track;

//-----------------------------------------------------------------------------------------------------------
B4sHandler::B4sHandler(Track::Tracklist* tlist) : QXmlDefaultHandler() {
    assert(tlist != 0);
    fTracklist = tlist;
    fCurTrack = 0;
    fCharBuffer = "";
    fCounter = 0;
   DomoGlobals::logger->debug("B4sHandler::B4sHandler", "Construction done");
}
//-----------------------------------------------------------------------------------------------------------
B4sHandler::~B4sHandler(){
}
//-----------------------------------------------------------------------------------------------------------
bool B4sHandler::startDocument() {
   DomoGlobals::logger->debug("B4sHandler::startDocument", "Starting Document");
    return true;
}
//-----------------------------------------------------------------------------------------------------------
bool B4sHandler::endDocument() {
    DomoGlobals::logger->debug("B4sHandler::endDocument", "End of Document reached");
    return true;
}
//-----------------------------------------------------------------------------------------------------------
bool B4sHandler::startElement( const QString& namespaceURI, 
			    const QString& local, 
			    const QString& qName, 
			    const QXmlAttributes& attributes)
{
 /*
  <WinampXML>
   <playlist num_entries="161" label="Playlist 001">
      <entry Playstring="file:D:\done\music\_SCOOTER____THE_AGE_OF_LOVE.MP3">
        <Name>Scooter - The Age Of Love</Name>
        <Length>228624</Length>
      </entry> 
     </playlist>
    </WinampXML>
*/
    DomoGlobals::logger->debug("B4sHandler::startElement","Starting element named '" + qName + "'");
 
    QString tmp2;
    QString tmp = namespaceURI; //to get rid of compiler warnings
    tmp = local; //to get rid of compiler warnings
    QString name = "";
    QString value = "";
    //cout<<"start element is '" + qName + "' local is '" + local + "'\n";
    
    if (qName == "WinampXML") {
	//do nothing
    } else if (qName == "playlist") {
	//do nothing
    } else if (qName == "entry") {
	fCurTrack = new Track();
	for(int i = 0; i < attributes.length(); ++i){
	    name = attributes.qName(i);
	    value = attributes.value(i);
	    
	    if (name == "Playstring"){
		int j = value.find(':', 0); //find first occurence of ':' character
		if (j < 0) {
		    DomoGlobals::logger->debug("B4sHandler::startElement","returning false,  j is: " + tmp2.setNum(j) );
		    return false;
		}
		value = value.remove(0,j+1); //strip "file:"
		fCurTrack->setPath( getPathFromName(value) );
		fCurTrack->setName( getNameFromPath(value) );
		//cout<<"in entry, path: " << fCurTrack->getPath() << " name: " << fCurTrack->getName() << "\n";
	    } else {
		assert(false);
	    }
	}
    } else if (qName == "Name") {
	//do nothing
    } else if (qName == "Length") {
	//do nothing
    } else {
	DomoGlobals::logger->debug("B4sHandler::startElement","Unknown element, qname is: " + qName);
	return false;
    }
    return true;
}
//-----------------------------------------------------------------------------------------------------------
bool B4sHandler::endElement( const QString& uri, const QString& localName, const QString& qName ){
    
    DomoGlobals::logger->debug("B4sHandler::endElement","Ending element named '" + qName + "'");
    
    QString tmp = uri; //to get rid of compiler warnings
    tmp = localName; //to get rid of compiler warnings
    
    if (qName == "WinampXML") {
	//do nothing
    } else if (qName == "playlist") {
	//do nothing
    } else if (qName == "entry") {
	assert(fTracklist != 0);
	assert(fCurTrack != 0);
	++fCounter;
	fCurTrack->setTrackNo(fCounter);
	fTracklist->push_back(fCurTrack);
	fCurTrack = 0;
	//cout<<"entry done\n";
    } else if (qName == "Name") {
	assert(fCurTrack != 0);
	QString name = fCharBuffer;
	//cout<<"end name, fCharBuffer: " << fCharBuffer << "\n";
	QString artist = "";
	QString title = "";
	QString comment = "";
	splitTrackname(name, artist, title, comment);
	fCurTrack->setArtist(artist);
	fCurTrack->setTitle(title);
	fCurTrack->setComment(comment);
    } else if (qName == "Length") {
	assert(fCurTrack != 0);
	QString value = fCharBuffer;
	fCurTrack->setLength( value.toInt()/1000 );
	//cout<<"end length, fCharBuffer: " << fCharBuffer << "\n";
    } else {
	DomoGlobals::logger->debug("B4sHandler::startElement","returning false, end element qname is: " + qName);
	return false;
    }

    return true;
}
//-----------------------------------------------------------------------------------------------------------
bool B4sHandler::characters ( const QString & ch ){
    fCharBuffer =  ch.stripWhiteSpace();
    //cout<<"CharBuffer is '" << fCharBuffer << "'\n";
    return true;
}
//-----------------------------------------------------------------------------------------------------------
