/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/

#include "musicexpression.h"
#include "consistencychecker.h"
#include "assert.h"
#include "skiptrack.h"
#include "splittrackdiag.h"
#include "metainfo.h"
#include "stringmanip.h"
#include "qdialog.h"
#include "fileio.h"
#include "domoglobals.h"
#include "filechooser.h"
#include "qlineedit.h"
#include "qcheckbox.h"
#include "qradiobutton.h"
#include "stringmanip.h"

using TrackSpace::Track;
using FileSpace::File;
using RcManSpace::RcMan;
//-----------------------------------------------------------------------------------------------------------
ConsistencyChecker::ConsistencyChecker(Log* logger, RcMan* rcman, Settings* settings) {
    assert(logger != 0);
    assert(rcman != 0);
    assert(settings != 0);
    fLogger = logger;
    fRcMan = rcman;
    fSettings = settings;
}
//-----------------------------------------------------------------------------------------------------------
ConsistencyChecker::~ConsistencyChecker() {
}
//-----------------------------------------------------------------------------------------------------------
bool ConsistencyChecker::splitFilename(Track* t, ConsistencyProp& properties) {
	assert(t != 0);
	QString title = "";
	QString artist = "";
	QString comment = "";
	
	SplitTrackDiag* dialog = new SplitTrackDiag();
	dialog->setProperties(properties);
	
	dialog->trackLine->setText(t->absPath());
	dialog->customArtist->setText(t->getArtist());
	dialog->customTitle->setText(t->getTitle());
	dialog->customComment->setText(t->getComment());
	
	dialog->enableTrm();
	
	Track* trk = new Track();
	trk->setName( t->getName() );
	trk->setPath( t->getPath() );
	MetaInfo::getMetaInfo(trk, true);
	artist = trk->getArtist();
	title = trk->getTitle();
	comment = trk->getComment();
	
	if (empty(artist)) artist = t->getArtist();
	if (empty(title)) title = t->getTitle();
	if (empty(comment)) comment = t->getComment();
	
	dialog->metaArtist->setText(artist);
	dialog->metaTitle->setText(title);
	dialog->metaComment->setText(comment);

	if((properties.fApplyToNone) || (properties.fApplyToAllInAlbum && !properties.fSameAlbum)) {
	    if (dialog->exec() == QDialog::Rejected) {
		delete dialog;
		delete trk;
		return false;
	    }
	}
	    
	if (dialog->custom->isChecked()) {
		artist = dialog->customArtist->text();
		title = dialog->customTitle->text();
		comment = dialog->customComment->text();
	} else if (dialog->meta->isChecked()) {
		artist = dialog->metaArtist->text();
		title = dialog->metaTitle->text();
		comment = dialog->metaComment->text();
	} else if (dialog->trm->isChecked()) {
		artist = dialog->trmArtist->text();
		title = dialog->trmTitle->text();
		comment = dialog->trmComment->text();
	} else if (dialog->onExpression->isChecked()) {
	    QString exp = dialog->expression->text();
	    Track* tk = MusicExpression::makeTrack(exp,stripExtension(t->getName()));
	    artist = tk->getArtist();
	    title = tk->getTitle();
	    comment = tk->getComment();
	    
	    if (tk->getYear() > 0) t->setYear(tk->getYear());
	    if (tk->getTrackNo() > 0) t->setTrackNo(tk->getTrackNo());
	    if (!empty(tk->getGenre())) t->setGenre(tk->getGenre());
	    
	    delete tk;
	} else {
	    QString artist = t->getName();
	    if (dashCount(artist) < dialog->getDashNum()) {
		fLogger->warning("ConsistencyChecker::splitFilename",
			      "To large dash count specified, splitting on first instead (artist = '" + artist + "')");
		splitTrackname(t->getName(),artist,title,comment, 1);
	    } else {
		splitTrackname(t->getName(),artist,title,comment, dialog->getDashNum());
	    }
	}
	
	t->setArtist(artist);
	t->setTitle(title);
	if (comment != "") t->setComment(comment);
	
	if (dialog->renameFile()) {
	    QString newname = "";
	    QString oldname = t->getPath() + DomoGlobals::DIR_SEP + t->getName();
	    QString ext = getExtension( t->getName() );
	    
	    if (comment != "") {
		newname = t->getPath() + DomoGlobals::DIR_SEP + artist + " - " + title + " (" + comment + ")." + ext;
	    } else {
		newname = t->getPath() + DomoGlobals::DIR_SEP + artist + " - " + title + "." + ext;
	    }
	    
	    if ( FileIO::renameFile( oldname,  newname ) ) {
		fLogger->info("ConsistencyChecker::splitFilename",
			      "File '" + oldname + "' renamed to '" + newname + "'");
	    } else {
		fLogger->error("ConsistencyChecker::splitFilename","Rename of file '" + oldname + "' failed...");    
	    }
	}
	
	dialog->getProperties(properties);
	delete dialog;
	delete trk;
	return true;
}
//-----------------------------------------------------------------------------------------------------------
bool ConsistencyChecker::unknownTrackField(Track* t, ConsistencyProp& properties) {
    properties.fEnableSplitMode = false;
    return splitFilename(t,properties); 
}
//-----------------------------------------------------------------------------------------------------------
bool ConsistencyChecker::skipTrack(const Track* t) {
    if (t == 0) {
	fLogger->error("ConsistencyChecker::skipTrack","Track is 0, serious error!!");
	return false;
    }
    
    SkipTrack* dialog = new SkipTrack();
    dialog->setTrack(t);
    int res = dialog->exec();
    delete dialog;
    
    if (res == 1) {
	QString path = t->absPath();
	if (FileIO::deleteFile(path)) {
	    fLogger->info("ConsistencyChecker::skipTrack","'" + path + "' deleted");
	} else {
	    fLogger->warning("ConsistencyChecker::skipTrack","Delete '" + path + "' failed");
	}
	return true;
    } else if (res == 2) {
	return true;
    } else if (res == 3) {
	return false;
    } else {
	fLogger->error("ConsistencyChecker::skipTrack","Invalid return code!");
	return false;
    }
    
}
//-----------------------------------------------------------------------------------------------------------
bool ConsistencyChecker::makeConsistent(Track::Tracklist& tlist, QProgressBar* progressBar) {
    if (fSettings->getBool(Settings::fSkipConsCheck)) {
	fLogger->warning("ConsistencyChecker::makeConsistent","Skipping consistency check");
	return true;
    }
    
    ConsistencyProp properties;
    properties.fRenameFile = fSettings->getBool(Settings::fAutoRename);

    progressBar->reset();
    progressBar->setTotalSteps(tlist.size());
    QString s1 = "";
    QString s2 = "";
    QString s3 = "";
    QString prevAlbum = "";
    QString curAlbum = "";
    
    fLogger->info("ConsistencyChecker::makeConsistent","Checking consistency...");
    Track::TrackIterator TI = tlist.begin();
    Track::TrackIterator end = tlist.end();
   
    if(TI != end) prevAlbum = (*TI)->getAlbum();
    
    int i = 1;
    while (TI != end) {
	curAlbum = (*TI)->getAlbum();
	properties.fSameAlbum = ((prevAlbum == curAlbum) && !empty(curAlbum));
	if(!properties.fSameAlbum) {
	    if(!empty(curAlbum)) fLogger->debug("ConsistencyChecker::makeConsistent","Changing running album from '" 
			   + prevAlbum + "' to '" + curAlbum + "'");
	    prevAlbum = curAlbum;
	} else {
	    fLogger->debug("ConsistencyChecker::makeConsistent","Still in the same album '" + prevAlbum + "'");   
	}
	
	if((properties.fApplyToNone) || (properties.fApplyToAllInAlbum && !properties.fSameAlbum)) {
	    properties.fSkipFile = false;
	    properties.fDoNothing = true;
	}
	
	if (! fSettings->getBool(Settings::fSkipQualityCheck)) {
	    //check for very short, very long, low bitrate tracks
	    if ((*TI)->fishyQuality()) {
		properties.fSkipFile = this->skipTrack(*TI);
	    }	
	} else {
	    fLogger->warning("ConsistencyChecker::makeConsistent","Skipping low quality check");
	}
	
	if (properties.fSkipFile == false) {
	    //get the number of '-' in the filename
	    i = dashCount((*TI)->getName());
	    if (i > 1){
		//get the number of visible dashes (visible = not enclosed by round/square brackets)
		int k = visibleDashCount((*TI)->getName());
		if (k < 1) {
		    //no visible dashes -> unkown artist (eg: 03 - My Song (Ice-T remix) )
		    if ( !unknownTrackField(*TI,properties) ) return false;
		} else if (k == 1) {
		    //only one visible dash, split on it
		    s1 = ""; s2 = ""; s3 = "";
		    splitOnVisibleDash((*TI)->getName(), s1, s2,s3,1);
		    (*TI)->setArtist(s1);
		    (*TI)->setTitle(s2);
		    (*TI)->setComment(s3);
		} else {
		    if (k == 2) {
			//two visible dashes
			s1 = ""; s2 = ""; s3 = "";
			//try to split on the second one
			splitOnVisibleDash((*TI)->getName(), s1, s2,s3,2);
			//if we are lucky the first part turns out to be a hypened artist (eg: Ice-T - Gangsta)
			if ( fRcMan->containsHyphenedArtist(s1) ) {
			    //if so, no need to consult the user
			    (*TI)->setArtist(s1);
			    (*TI)->setTitle(s2);
			    (*TI)->setComment(s3);
			} else {
			    //user intervention needed
			    properties.fRememberArtist = true;
			    properties.fEnableSplitMode = true;
			    if ( !splitFilename(*TI,properties) ) return false;
			    rememberArtist(*TI, properties);
			}
		    } else {
			//more than two visible dashes, user intervention needed
			properties.fEnableSplitMode = true;
			if ( !splitFilename(*TI,properties) ) return false;
			rememberArtist(*TI, properties);
		    }
		}
	    } else if (i == 0) {
		//no dashes at all, need to ask user
		if ( !unknownTrackField(*TI,properties) ) return false;
	    } else if (i == 1) {
		/*only one dash, don't automatically split on this one dash to
		to prevent tracks like "02 - This is the title" being added */
		QString name = (*TI)->getName();
		bool b = fRcMan->containsHyphenedArtist(name);
		
		if ( ! fRcMan->startsWithSafeArtist(name)) {
		    name = stripTrackNo(name);
		}
		
		//does name contain a hypended artist or is the dash part of the track no?
		if ((dashCount(name) == 0) || b) {
		    if ( !unknownTrackField(*TI,properties) ) return false;
		}
		
	    } else {
		assert(i >= 0);
	    }
	}	
	
	//did the user want to skip the track?
	if (properties.fSkipFile) {
	    //erase current track from the list
	    fLogger->info("ConsistencyChecker::makeConsistent","Skipping track '" + (*TI)->getName() + "'");
	    //Did the user want to delete the file
	    if(properties.fDeleteFile) {
		QString path = (*TI)->absPath();
		if (FileIO::deleteFile(path)) {
		    fLogger->info("ConsistencyChecker::makeConsistent","'" + path + "' deleted");
		} else {
		    fLogger->warning("ConsistencyChecker::makeConsistent","Delete '" + path + "' failed");
		}
	    }
    	    delete (*TI);
	    TI = tlist.erase(TI);
	} else {
	    //try to strip the track number from the artist field
	    const QString artist = (*TI)->getArtist();
	    if ( ! fRcMan->startsWithSafeArtist(artist)) {
		(*TI)->setArtist(stripTrackNo(artist));
	    } else {
		fLogger->debug("ConsistencyChecker::makeConsistent","Track number not stripped from: " + artist);
	    }
	    ++TI;
	}
	progressBar->setProgress(i);
	++i;
    }
    progressBar->reset();
    return true;
}
//-----------------------------------------------------------------------------------------------------------
void ConsistencyChecker::rememberArtist(const Track* trk, ConsistencyProp properties) {
    assert(trk != 0);
    if (properties.fRememberArtist) {
	QString s1 = trk->getArtist();
	if (dashCount(s1) == 1) {
	    fRcMan->addHyphenedArtist(s1);
	    fLogger->info("ConsistencyChecker::rememberArtist",
			  "Added hyphened artist '" + s1 + "'");
	} else {
	    fLogger->warning("ConsistencyChecker::rememberArtist",
			     "Artist '" + s1 + "' doesn't contain 1 dash, artist not added");
	}
    }
}
//-----------------------------------------------------------------------------------------------------------
