/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/
#ifndef INC_DBASEBACKUP_H
#include "dbasebackup.h"
#define INC_DBASEBACKUP_H
#endif

#ifndef INC_DOMOGLOBALS_H
#include "domoglobals.h"
#define INC_DOMOGLOBALS_H
#endif

#include <iostream>
#include "qdir.h"
#include "qobject.h"

using std::cout;
//-----------------------------------------------------------------------------------------------------------
DbaseBackup::DbaseBackup(const QString user, 
			 const QString pwd, 
			 const QString dbase, 
			 const QString file,
			 const QString dir,
			 const QString switches )
{
    fFile = file;
    fWriter = new WriterTxt(fFile);
    QString s = "-- Created by DOMO v" + DomoGlobals::DOMO_VERSION + "\n"
		+ "-- To use this file to restore you database:\n"
		+ "--       Create a new database [database]\n"
		+ "--       Execute \"mysql [database] < [this file]\"\n"
		+ "--\n"
		+ "-- (C) GoriTech Domo is available for free under the GNU GPL at http://domo.sf.net/\n"
		+ "-- Please send all bugs/comments/requests to Dirk Gorissen <" + DomoGlobals::AUTHOR_MAIL+">\n"
		+ "--\n\n";

    fWriter->writeHeader(s);

    fProc = new QProcess( QString("mysqldump") );
    
      if (switches.isEmpty()) {
	if (dir != "") fProc->setWorkingDirectory( QDir(dir) );
	fProc->addArgument( "-p" + pwd);
	fProc->addArgument( "-u" + user );
	fProc->addArgument( "--add-drop-table" );
	fProc->addArgument( dbase );
    } else {
	fProc->addArgument( switches);
    }     
    
    connect( fProc, SIGNAL(processExited()),this,SLOT(backupFinished() ));
    connect( fProc, SIGNAL(readyReadStdout()),this, SLOT(readFromStdout()) );
}

//-----------------------------------------------------------------------------------------------------------
DbaseBackup::~DbaseBackup() {
    delete fProc;
}
//-----------------------------------------------------------------------------------------------------------
bool DbaseBackup::start() {
    if ( !fProc->start() ) {
	DomoGlobals::logger->error("DbaseBackup::start","Backup process failed to start");
	return false;
    } else {
	DomoGlobals::logger->info("DbaseBackup::start","Backup process started");
	return true;
    }
}

//-----------------------------------------------------------------------------------------------------------
void DbaseBackup::readFromStdout() {
    QString s(fProc->readStdout());
    
    fWriter->writeLn(s);
    fWriter->flush();
}
//-----------------------------------------------------------------------------------------------------------
void DbaseBackup::backupFinished() {
    DomoGlobals::logger->info("DbaseBackup::backupFinished","Backup process finished");
}
//-----------------------------------------------------------------------------------------------------------
