/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/
#ifndef INC_IOEXCEPTION_H
#include "ioexception.h"
#define INC_IOEXCEPTION_H
#endif

#ifndef INC_DIRSCANNER_H
#include "dirscanner.h"
#define INC_DIRSCANNER_H
#endif

#ifndef INC_STRINGMANIP_H
#include "stringmanip.h"
#define INC_STRINGMANIP_H
#endif

#ifndef INC_METAINFO_H
#include "metainfo.h"
#define INC_METAINFO_H
#endif

#ifndef INC_FILEIO_H
#include "fileio.h"
#define INC_FILEIO_H
#endif

#include "musicexpression.h"
#include "domoglobals.h"
#include "readerfactory.h"
#include "qdatetime.h"
#include "qstringlist.h"
#include "assert.h"
#include "qregexp.h"
#include <iostream>

using  std::cout;
using FileSpace::File;
using DirScannerSpace::DirScanner;
using RcManSpace::RcMan;
using ReaderSpace::ReaderFactory;
using ReaderSpace::IReader;

//-----------------------------------------------------------------------------------------------------------
DirScanner::DirScanner(RcMan* rc, Settings* set, const QString path) {
	assert(rc != 0);
	assert(set != 0);
	
	fPath = path;
	fRcMan = rc;
	fSettings = set;
}
//-----------------------------------------------------------------------------------------------------------
DirScanner::~DirScanner(){
    //pointer fRcMan not owned by dirscanner
}
//-----------------------------------------------------------------------------------------------------------
QString DirScanner::getPath() const {
    return fPath;
}
//-----------------------------------------------------------------------------------------------------------
void DirScanner::setPath(const QString s) {
    fPath = s;
}
//-----------------------------------------------------------------------------------------------------------
void DirScanner::scanDir(QString& path, QString& filter, QFileInfoList& flist, const bool recurse, const bool incDirs) {
    QDir d(path);
   
    if (!d.exists(path)) throw (new IoException("DirScanner::scanDir", "Path '" + QString(path) + "' does NOT exist"));
    
    d.setPath(path);
    const QFileInfoList* dirList;
    const QFileInfoList* tmpList;
    QString dirAbsPath="";
    bool b = true;
    
    if (incDirs) {
	tmpList = d.entryInfoList("*. " + filter, QDir::Files|QDir::Dirs, QDir::Name);	
    }else {
	tmpList = d.entryInfoList(filter, QDir::Files, QDir::Name);	//add files in directory to list
    }
    
    QFileInfoListIterator it( *tmpList );
    QFileInfo *fi2;
    QString fname = "";
    while ( (fi2 = it.current()) != 0 ) {
	if (!fi2->isDir()) {
	    fname = fi2->fileName();
	} else {
	    fname = fi2->dirPath();
	    // f << "dir found: " << fname << "\n";
	}
	
	if ((fname != path) && (fname != "..") && (fname != ".")) {
	    QFileInfo* p = new QFileInfo(*fi2);
	    p->setCaching(false);
	    flist.append(p);
	    //f << "     added file: " << fname << "\n";
	}
	++it;
    }
    
    
    dirList = d.entryInfoList(QDir::Dirs, QDir::Name); 	//get the list of sub directories
    if (!recurse||(d.count() <= 0)||(dirList == 0)||(dirList->isEmpty())) {//'path' contains sub dirs
	//f << "recursion stopped" << "\n";
	//recursion stops		
    } else {
	QFileInfoListIterator it3( *dirList );
	QFileInfo *fi;
	
	while (( (fi = it3.current()) != 0 ) && b) {
	    if ((fi->fileName() != ".") && (fi->fileName() != "..")  && (fi->isDir()) ) {
		dirAbsPath = fi->absFilePath();					//get the absolute path of the directory
		d.setPath(dirAbsPath);						//change to that directory
		if (b) {
		    //f << "going to scan: " << dirAbsPath << "\n";
		    this->scanDir(dirAbsPath, filter, flist, recurse, incDirs);	    //scan contents (recurse)	
		}
	    }
	    ++it3;
	}
    }
	              
}
//-----------------------------------------------------------------------------------------------------------
File::Filelist DirScanner::indexDirectory(const QString mask, const bool recurse, const bool incDirs){
    QFileInfoList flist;
    File::Filelist fl;
    QString ext = "";
    
    QString pth = fPath;
    QString msk = mask;
    
    this->scanDir(pth, msk, flist, recurse, incDirs);
    
    
    QFileInfoListIterator it( flist );
    QFileInfo fi;
    
    while ( it.current() != 0 ) {
	fi = (*it.current());	
	ext = fi.extension();
	File* f = new File(fi.fileName(), fRcMan->getExtDesc(ext),fi.size());
	if (f == 0) {
	    //throw memAllocException();
	} else {
	    f->setHidden(false);
	    //f->setArchive(fi.);
	    //f->setSystem(fia[i].IsSystem());
	    //f->setVolumeLabel(false); //still has to be fixed
	    //f->setCompressed(fia[i].IsCompressed());
	    //f->setTemporary(fia[i].IsTemporary());
	    
	    f->setCreated(fi.created().toString("yyyy.MM.dd  h:m:s"));
	    f->setModified(fi.lastModified().toString("yyyy.MM.dd  h:m:s"));
	    f->setAccessed(fi.lastRead().toString("yyyy.MM.dd  h:m:s"));
	    
	    pth = fi.absFilePath();
	    f->setPath(pth);
	    f->setDirectory(fi.isDir());
	    f->setReadOnly(fi.isReadable());
	    
	    fl.push_back(f);
	}
	++it;
    }
    flist.clear();
	return fl;
}
//-----------------------------------------------------------------------------------------------------------
QString DirScanner::makeAlbum(const QString&trackPath, const QString& scanPath) {
    QString s1 = (QDir::convertSeparators(trackPath));
    QString s2 = (QDir::convertSeparators(scanPath));
    
    /* may not happen in all cases
    QChar lastChar = s2[s2.length()-1];
    if (QString(lastChar) == DomoGlobals::DIR_SEP) {
	//pop '/' (or '\') if scanpath ends with a directory separator
	//to make sure subtraction always works
	s2 = s2.left(s2.length()-1);
    } */

    QString s3 = subtract(s1,s2);
   
    //DomoGlobals::logger->debug("DirscannerSpace::makeAlbum","Subtract: '" + s1 + "' - '" + s2 + "' = '" + s3 + "'");
    
    QString album = MusicExpression::fitToPattern(s3,
				  fSettings->get(Settings::fAlbumSrcPattern),
				  fSettings->get(Settings::fAlbumDestPattern));
    
    if(s3 != "") DomoGlobals::logger->debug("DirscannerSpace::makeAlbum",
					    "String in '" + s3 + "' album out '" + album + "'");
   
    return cleanFilename(album);
}
//-----------------------------------------------------------------------------------------------------------
Track::Tracklist DirScanner::indexMusicDirectory(const QString mask, const bool recurse, const bool incDirs, QProgressBar* pBar){
    QFileInfoList flist;
    Track::Tracklist fl;
    QString ext = "";
    QString tmp = "";
    QString filename = "";
    
    QString pth = fPath;
    QString msk = mask;
    
    this->scanDir(pth, msk, flist, recurse, incDirs);
    
    QFileInfoListIterator it( flist );
    QFileInfo fi;
    
    pBar->reset();
    pBar->setTotalSteps(flist.count());
    int pi = 0;	
    int i = 1;
    while ( it.current() != 0 ) {
	fi = (*it.current());	
	ext = getExtension(fi.fileName());
	
	filename = fi.fileName();
	
	//cout<<"Current file: '" << filename << "'\n";
	
	if ( ! fRcMan->startsWithSafeArtist(filename)) {
	   filename = stripTrackNo(filename); //strip away the portion of the filename indicating the track number
	}
	Track* trk = TrackSpace::makeTrack(filename); //trk now (possibly) contains a non existing filename (if the track number was stripped)
	trk->setName(fi.fileName());  //reset the filename to its original value
	
	if (trk == 0) {
	    assert(false);
	} else {
	    if (!fi.isDir()) {
		trk->setName(fi.fileName());
		trk->setSize(fi.size());
		trk->setDescription(fRcMan->getExtDesc(ext));
		trk->setCreated(fi.created().toString("yyyy.MM.dd  h:m:s"));
		trk->setModified(fi.lastModified().toString("yyyy.MM.dd  h:m:s"));
		trk->setAccessed(fi.lastRead().toString("yyyy.MM.dd  h:m:s"));
		pth = fi.dirPath();
		pth = QDir::convertSeparators(pth);
		trk->setPath(pth);
		trk->setDirectory(fi.isDir());
		trk->setReadOnly(fi.isReadable());
		
		tmp = makeAlbum(trk->getPath(),fPath);
	
		trk->setAlbum(tmp);
		
		trk->setTrackNo(i);
		++i;
		
		if (fi.size() > 0) {
		    MetaInfo::getMetaInfo(trk,false);
		    QString genre = fRcMan->resolveGenre(trk->getGenre().toInt());
		    trk->setGenre(genre);
		}
		//trk->setMood();
		//trk->setScore();
		//trk->setArchive(fi.);
		//trk->setSystem(fia[i].IsSystem());
		//trk->setVolumeLabel(false); //still has to be fixed
		//trk->setCompressed(fia[i].IsCompressed());
		//trk->setTemporary(fia[i].IsTemporary());			
		//trk->setHidden(false);
	    } else {
		trk->setDirectory(true);
		trk->setName(subtract(fi.dirPath(),fPath));
	    }		
	    
	    if (trk->getSize() >= 0) {
		fl.push_back(trk);
	    }
	    
	}
	++it;
	++pi;
	pBar->setProgress(pi);
    }
    
    flist.clear();
    
	return fl;
}
//-----------------------------------------------------------------------------------------------------------
void DirScanner::getTracksFromSource(const QString fname,Track::Tracklist& tlist, QProgressBar* progressBar) {
    
    if (!FileIO::exists(fname)) {
	throw (new IoException("DirScanner::getTracksFromSource","Path '" + fname + "' does not exist"));
    }    
    
    bool directory = FileIO::isDirectory(fname);
    
    if (directory) {
	getTracksFromDir(fname, tlist, progressBar);
    } else {
	getTracksFromFile(fname, tlist);
    }
}
//-----------------------------------------------------------------------------------------------------------
void DirScanner::getTracksFromDir(const QString fname,Track::Tracklist& tlist, QProgressBar* progressBar) {
   setPath(fname);
   tlist = indexMusicDirectory(DomoGlobals::MUSIC_MASK,
			       fSettings->getBool(Settings::fRecurseDirs),
			       false,
			       progressBar);
}
//-----------------------------------------------------------------------------------------------------------
void DirScanner::getTracksFromFile(const QString fname,Track::Tracklist& tlist) {
    ReaderFactory* factory = new ReaderFactory();
    
    IReader* reader = factory->getReader(fname);
    assert(reader != 0);
    
    reader->read(tlist);
    delete reader;
    delete factory;
}
//-----------------------------------------------------------------------------------------------------------
