/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/

#ifndef DOMOMAINWINDOW_H
#define DOMOMAINWINDOW_H

#ifndef INC_SETTINGS_H
#include "settings.h"
#define INC_SETTINGS_H
#endif

#ifndef DOMOMAINWINDOWBASE_H
#include "domomainwindowbase.h"
#define DOMOMAINWINDOWBASE_H
#endif

#ifndef INC_COLLECTION_H
#include "collection.h" 
#define INC_COLLECTION_H
#endif

#ifndef INC_DBASE_H
#include "dbase.h"
#define INC_DBASE_H
#endif

#ifndef INC_COMPARE_H
#include "compare.h"
#define INC_COMPARE_H
#endif

#ifndef INC_RCMAN_H
#include "rcman.h"
#define INC_RCMAN_H
#endif

#ifndef INC_DIRSCANNER_H
#include "dirscanner.h"
#define INC_DIRSCANNER_H
#endif

#include <qerrormessage.h>
#include "guilog.h"
#include "consistencychecker.h"

using CompareSpace::Compare;
using DbaseSpace::Dbase;
using std::ostringstream;
using CollectionSpace::Collection;
using DirScannerSpace::DirScanner;

class DOMOMainWindow : public DOMOMainWindowBase
{
    Q_OBJECT
private:
    Collection* fCurCollection;
    Dbase* fDB;
    Log* fLogger;
    Settings* fSet;
    Compare* fCompare;
    DirScanner* fDirScan;
    RcMan* fRcMan;  //Resource manager
    ConsistencyChecker* fConsistencyChecker;
    void initWidgets();
    bool confirm(const QString msg);
    void updateCollectionList();
    
public:
    DOMOMainWindow( QWidget* parent = 0, const char* name = 0, WFlags f = WType_TopLevel );
    
    ~DOMOMainWindow();
    void setDatabase(Dbase* db);
    void backupDatabase();
    
protected slots:
    virtual void addCD();
    virtual void compare();
    virtual void createCollection();
    virtual void query();
    virtual void setActiveCollection( const QString & );
    virtual void exportToFile();
    virtual void trackInfoClicked();
    virtual void clearCompareResults();
    virtual void useCompareSelection();
    virtual void getStats();
    virtual bool deleteCD();
    virtual void preferencesDialog();
    virtual void lookupCD();
    virtual void deleteCurCollection();
    virtual void cleanDir();
};

#endif
