/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/
#include "qapplication.h"
#include "qfile.h"
#include "qradiobutton.h"
#include "qerrormessage.h"
#include "qsqldatabase.h"
#include "qpixmap.h"
#include "qsplashscreen.h" 
#include "qobject.h" 

#ifndef INC_IOEXCEPTION_H
#include "ioexception.h"
#define INC_IOEXCEPTION_H
#endif

#ifndef INC_DBASEEXCEPTION_H
#include "dbaseexception.h"
#define INC_DBASEEXCEPTION_H
#endif

#ifndef FILECHOOSER_H
#include "filechooser.h"
#define FILECHOOSER_H
#endif

#ifndef SETUPDIALOG_H
#include "setupdialog.h"
#define SETUPDIALOG_H
#endif

#ifndef DOMOMAINWINDOW_H
#include "domomainwindow.h"
#define DOMOMAINWINDOW_H
#endif

#ifndef INC_DOMOGLOBALS_H
#include "domoglobals.h"
#define INC_DOMOGLOBALS_H
#endif

#ifndef INC_DBASE_H
#include "dbase.h"
#define INC_DBASE_H
#endif

#ifndef LANGDIALOG_H
#include "langdialog.h"
#define LANGDIALOG_H
#endif

#ifndef INC_SETTINGS_H
#include "settings.h"
#define INC_SETTINGS_H
#endif

#ifndef INC_STRINGMANIP_H
#include "stringmanip.h"
#define INC_STRINGMANIP_H
#endif

#include "dirscanner.h"
#include "dbasefactory.h"
#include "qtranslator.h"
#include "musicexpression.h"
#include <iostream>

using std::cout;
using DbaseSpace::Dbase;
using DbaseSpace::DbaseFactory;

void test(QString s, QString s2, QString s3) {
    cout << "in: '" << s <<"'\n";
 //   cout << "in2: '" << s2 <<"'\n";
   // cout << "in3: '" << s3 <<"'\n";
//    QString out = MusicExpression::fitToPattern(s,s2,s3);
    QString out = largeCase(s);
   cout << "after: '" << out <<"'\n";
//   cout << "after: '" << DirScannerSpace::translateToRegex(s) <<"'\n";
}

int main( int argc, char** argv )
{
    
   cout <<"\nDomo comes with ABSOLUTELY NO WARRANTY.  This is free software,"
	"\nand you are welcome to redistribute it under certain conditions; for details,"
	"\nsee the COPYING file or the About dialog\n\n";
   
   
    QApplication app( argc, argv );
    QTranslator translator( 0 );
    
    try {
	
	if (!QFile::exists(DomoGlobals::DOMO_DIR + DomoGlobals::SETTINGS_FNAME)) {
	    try{
		DomoGlobals::logger = new Log("domo.install.log");
		DomoGlobals::logger->open();
		DomoGlobals::logger->info("main","All log information is logged to '~/.domo/domo.install.log'");
	    } catch (IoException* e) {
		cout<<"Failed to instantiate/open logger, exception is: " << e->msg() << "\n";
	    }
	    
	    LangDialog* langDiag = new LangDialog();
	    langDiag->exec();
	    QString file = langDiag->getLangFile();
	    delete langDiag;
	    
	    if (file != "default") {
		//look first in the current directory, then in the home directory and finally in the config directory
		QString path = QDir::currentDirPath() +  DomoGlobals::DIR_SEP 
			       + DomoGlobals::LANGUAGE_DIR +  DomoGlobals::DIR_SEP + file;
		if (!QFile::exists(path)) path = DomoGlobals::DOMO_DIR
						 + DomoGlobals::LANGUAGE_DIR + DomoGlobals::DIR_SEP + file; 
		if (!QFile::exists(path)) path = DomoGlobals::CONFIG_DIR 
						 + DomoGlobals::LANGUAGE_DIR + DomoGlobals::DIR_SEP + file;
		
		if (translator.load( path )) {
		    DomoGlobals::logger->info("main","Loading language file '" + QString(path) + "'");
		    app.installTranslator( &translator );
		} else {
		    DomoGlobals::logger->info("main","Loading language file '" + QString(path) + "' failed");
		}
	    }
	    
	    SetupDialog* dialog = new SetupDialog();
	    dialog->setLangFile(file);
	    app.setMainWidget(dialog);
	    
	    DomoGlobals::logger->warning("main",DomoGlobals::SETTINGS_FNAME + " is missing, running setup Wizard...");	
	    
	    dialog->exportDir->setFileName(DomoGlobals::DOMO_DIR + "export");
	    dialog->cdromDir->setFileName("/dev/cdrom");
	    dialog->addDir->setFileName(DomoGlobals::DOMO_DIR);
	    dialog->show();
	    app.exec();
	    int res = dialog->res();
	    
	    if (res == QDialog::Accepted) {
		bool b = dialog->install();
		if (b) {
		    DomoGlobals::logger->info("main","Installation successfull!! Now re-run Domo\n\n"
					      "Thank you for chosing Domo!");
		} else {
		    DomoGlobals::logger->error("main","Installation failed, see above errors.\n"
					       " -- Please send a bugreport to <" +DomoGlobals::AUTHOR_MAIL+"> if necessary");
		    QFile::remove(DomoGlobals::DOMO_DIR + DomoGlobals::SETTINGS_FNAME);
		}
	    } else {
		DomoGlobals::logger->info("main","Domo setup wizard cancelled");
	    }
	    delete dialog;
	} else {
	    try {
		DomoGlobals::logger = new Log(DomoGlobals::LOG_FILE);
		DomoGlobals::logger->open();
		DomoGlobals::logger->info("main","All log information is logged to '" + DomoGlobals::LOG_FILE + "'");
	    } catch (IoException* e) {
		cout<<"Failed to instantiate/open logger, exception is: " << e->msg() << "\n";
	    }
	    
    	    // test(argv[1],argv[2], argv[3]);
	    // return 0;

	    DomoGlobals::logger->info("main",DomoGlobals::SETTINGS_FNAME + " found...");
	    
	    Settings* fs = new Settings(DomoGlobals::DOMO_DIR + DomoGlobals::SETTINGS_FNAME);
	    if (!fs->getSettings()) {
		DomoGlobals::logger->error("main","The file 'Domo.conf' is erronous, Please correct or delete this file and restart!");
		return -1;
	    }
	    
	    DomoGlobals::logger->setDebugLevel(fs->getInt(Settings::fDebugLevel));
	    DomoGlobals::logger->info("main","Settings loaded...");
	    
	    QPixmap pixmap( DomoGlobals::DOMO_DIR + DomoGlobals::SPLASHSCREEN );
	    QSplashScreen *splash = 0;
	    if (!pixmap.isNull()) {
		splash = new QSplashScreen( pixmap );
		splash->show();
		splash->message(QObject::tr("Creating database connections...") );
	    } else {
		DomoGlobals::logger->warning("main","Splashscreen file '" + DomoGlobals::SPLASHSCREEN 
					     + "' not found");
	    }
	    
	    QString dbDriver = fs->get(Settings::fDbaseDriver);
	    DbaseFactory* factory = new DbaseFactory();
	    Dbase* db = factory->createDbase(dbDriver);
	    delete factory;
	    
	    QString pass = decrypt(fs->get(Settings::fDbasePwd));
	    QString name = fs->get(Settings::fDbaseName);
	    QString user = fs->get(Settings::fDbaseUser);
	    QString host = fs->get(Settings::fDbaseHost);
	    
	    try {
		db->createConnections(name, user, pass, host);
	    } catch (DbaseException* e) {
		DomoGlobals::logger->error("main","Database connection failed bailing out, error is: '" + e->msg() + "'");
		delete fs;
		return -1;
	    }
	    
	    DomoGlobals::logger->info("main","Database connection created....");
	    if (splash != 0) splash->message(QObject::tr("Reading resource files..."));
	    if (splash != 0) splash->message(QObject::tr("Reading database..."));
	    
	    QString file = fs->get(Settings::fLanguageFile);
	    QString dir = fs->get(Settings::fLanguageDir);
	    if (file != "default") {
		file = dir + DomoGlobals::DIR_SEP + file;
		if (translator.load( file )) {
		    DomoGlobals::logger->info("main","Loading language file '" + file + "'");
		    app.installTranslator( &translator );
		} else {
		    DomoGlobals::logger->error("main","Loading language file '" + file + "' failed");
		}
	    }
	    
	    DOMOMainWindow mainwindow;
	    mainwindow.setDatabase(db);
	    app.setMainWidget(&mainwindow);
	    mainwindow.show();
	    
	    if (!pixmap.isNull()) {
		splash->finish( &mainwindow );
		delete splash;
	    }
	    
	    app.exec();
	    delete fs;
	}
	
    } catch(Exception* e) {
	cout<<"Unhandled exception: " << e->msg() << "\n";
	cout<<" --> Please send the above output to " + DomoGlobals::AUTHOR_MAIL + "\n";
	return -1;
    }
	return 0;
}

