/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/
#include "metainfo.h"
#include "exception.h"
#include "stringmanip.h"
#include "assert.h"
#include "qstringlist.h"
#include "domoglobals.h"
#include <stdio.h>
#include <id3/tag.h>
#include <id3/misc_support.h>
#include <vorbis/vorbisfile.h>
#include <string>
using std::string;
//-----------------------------------------------------------------------------------------------------------
MetaInfo::MetaInfo() {
    
}
//-----------------------------------------------------------------------------------------------------------
MetaInfo::~MetaInfo() {
    
}
//-----------------------------------------------------------------------------------------------------------
void MetaInfo::getMetaInfo(Track* trk, const bool getArtistAndTitle) {
    assert(trk != 0);
    ContentType t = getContentType(trk->getName());

    switch (t) {
	case MP3: {
	    getMp3Info(trk, getArtistAndTitle);
	    break;
	}
	case OGG: {
	    getOggInfo(trk, getArtistAndTitle);
	    break;
	}
	case WMA:{
	    getWmaInfo(trk, getArtistAndTitle);
	    break;
	}
	case WAV:{
	    getWavInfo(trk, getArtistAndTitle);
	    break;
	}
	default: {
	    break;
	}
    }
}
//-----------------------------------------------------------------------------------------------------------
void MetaInfo::setMetaInfo(Track* trk) {
    assert(trk != 0);
    
}
//-----------------------------------------------------------------------------------------------------------
void MetaInfo::getID3Tag(const QString path,
			 QString& artist, 
			 QString& title, 
			 QString& comment,
			 QString& year,
			 QString& album,
			 QString& genre) {
    
    if (!QFile::exists(path)) return;
    
   QString tmp = "";
   
    ID3_Tag myTag;
    myTag.Link ( path );
    
    ID3_Frame* myFrame = myTag.Find(ID3FID_TITLE);
    if (myFrame != NULL) {
	tmp = ID3_GetTitle(&myTag);
	tmp = makeASCII(tmp);
    }
    title = tmp;
    tmp = "";
    
    myFrame = myTag.Find(ID3FID_LEADARTIST);
    if (myFrame != NULL) {
	tmp = ID3_GetArtist(&myTag);
	tmp = makeASCII(tmp);
    }
    
    artist = tmp;
    tmp = "";
    
    myFrame = myTag.Find(ID3FID_COMMENT);
    if (myFrame != NULL) {
	tmp = ID3_GetComment(&myTag);
	tmp = makeASCII(tmp);
    }
    comment = tmp;
    tmp = "";
    
    myFrame = myTag.Find(ID3FID_YEAR);
    if (myFrame != NULL) {
	tmp = ID3_GetYear(&myTag);
	tmp = makeASCII(tmp);
    }
    year = tmp;
    tmp = "";
    
    myFrame = myTag.Find(ID3FID_ALBUM);
    if (myFrame != NULL) {
	tmp = ID3_GetAlbum(&myTag);
	tmp = makeASCII(tmp);
    }
    album = cleanFilename(tmp);
    tmp = "";
    
    myFrame = myTag.Find(ID3FID_CONTENTTYPE);
    if (myFrame != NULL) {
	tmp = ID3_GetGenre(&myTag);
	tmp = makeASCII(tmp);
    }
    //genre is returned as '(83)' we only want the number
   QString s = ""; s += tmp[1]; s += tmp[2];  
   genre = s;
   tmp = "";
   
   myTag.Clear();
}
//-----------------------------------------------------------------------------------------------------------
void MetaInfo::getMpegHeader(const QString path, int& brate, int& freq, int& len, QString& mode, QString& version, QString& layer) {
   ID3_Tag* myTag = new ID3_Tag(path.latin1());
   
   const Mp3_Headerinfo* mp3Header = new Mp3_Headerinfo();
   mp3Header = myTag->GetMp3HeaderInfo();
   if (mp3Header != 0) {
       len = mp3Header->time;
       freq = (mp3Header->frequency);
       brate = (mp3Header->bitrate);
       
       switch (mp3Header->channelmode)  {
       case MP3CHANNELMODE_SINGLE_CHANNEL:
	   mode = "Single";
	   break;
       case MP3CHANNELMODE_DUAL_CHANNEL:
	   mode = "Dual";
	   break;
       case MP3CHANNELMODE_JOINT_STEREO:
	   mode = "Joint Stereo";
	   break;
       case MP3CHANNELMODE_STEREO:
	   mode = "Stereo";
	   break;
       default:
	   break;
       }
       
       switch (mp3Header->version) {
       case MPEGVERSION_2_5:
	   version = "MPEG2.5";
	   break;
       case MPEGVERSION_2:
	   version = "MPEG2";
	   break;
       case MPEGVERSION_1:
	   version = "MPEG1";
	   break;
       default:
	   break;
       }
       
       switch (mp3Header->layer) {
       case MPEGLAYER_III:
	   layer = "layer III\n";
	   break;
       case MPEGLAYER_II:
	   layer =  "layer II\n";
	   break;
       case MPEGLAYER_I:
	   layer =  "layer I\n";
	   break;
       default:
	   break;
       }
       
       //delete mp3Header -> causes prog to hang for some reason;
   }
   //cout<<"lenght: " << len <<" bitrate: "<<brate/1000 << " freq: " << freq << "\n";
   myTag->Clear();
   delete myTag;
}
//-----------------------------------------------------------------------------------------------------------
void MetaInfo::getMp3Info(Track* t, const bool getArtistAndTitle) {
    assert(t != 0);
    QString fname = t->getPath() + DomoGlobals::DIR_SEP + t->getName();
   //cout<<"filename: " << fname << "\n";   
   
   int brate = 0;
   int freq = 0;
   int len = 0;
   QString mode = "";
   QString version = "";
   QString layer = "";
   
   getMpegHeader(fname, brate, freq, len, mode, version, layer);
   
   //cout<<"lenght: " << len <<" bitrate: "<<brate/1000 << " freq: " << freq << "\n";
   t->setBitrate(brate/1000);
   t->setSamplingRate(freq);
   t->setLength(len);   
   
   QString path = fname;
   QString artist = ""; 
   QString title = "";
   QString comment = "";
   QString year = "";
   QString album = "";
   QString genre = "";
   
   getID3Tag(path, artist, title, comment, year, album, genre);
   
   if (comment.stripWhiteSpace().isEmpty() ) t->appendComment(comment);
   if ((album.stripWhiteSpace().isEmpty() ) && (t->getAlbum().isEmpty())) t->setAlbum(album);
   
   t->setYear(year);
   t->setGenre(genre);
   
   if (getArtistAndTitle) {
       t->setArtist(artist);
       t->setTitle(title);
   }

}
/*----------------------------------------------------------------------------------------------------------
The follwoing method contains highly borrowed code from:  
QMBTagger - Qt Based MusicBrainz MP3/OGG Tagger
Copyright (C) 2003  Ewen Cheslack-Postava */
//-----------------------------------------------------------------------------------------------------------
void MetaInfo::getOggInfo(Track* t, const bool getArtistAndTitle) {
    assert(t != 0);
    
    FILE* file = NULL;
    OggVorbis_File vf;

    //record the full filename so we will be able to use it for output
    QString filename = t->absPath();

    //open the file
    file = fopen(filename.latin1(), "r");

    //check that the file is in fact vorbis
    if(ov_open(file, &vf, NULL, 0) < 0) {
	DomoGlobals::logger->error("MetaInfo::getOggInfo()","ov_open failed: File '" 
				   + filename + "' probably isn't a proper Ogg Vorbis file");
	return;
    }

    vorbis_comment* vc = ov_comment(&vf, -1);
    
    if(vc == NULL) {
	DomoGlobals::logger->error("MetaInfo::getOggInfo()",
				   "Error getting vorbis_comment on file '" 
				   + filename + "', File must be corrupted");
	return;
    }
    
    QString title,album,artist,trackNo,version,
    performer,copyright,license,organization,description,
    genre,date,location,contact,isrc;
    
    for(int i = 0; i < vc->comments; i++) {
	QString thiscomment(vc->user_comments[i]);
	
	if (thiscomment.left(6).upper() == "TITLE=") {
	    title = thiscomment.right(thiscomment.length() - 6);
	} else if (thiscomment.left(6).upper() == "ALBUM=") {
	    album = thiscomment.right(thiscomment.length() - 6);
	} else if (thiscomment.left(7).upper() == "ARTIST=") {
	    artist = thiscomment.right(thiscomment.length() - 7);
	} else if (thiscomment.left(12).upper() == "TRACKNUMBER=") {
	    trackNo = thiscomment.right(thiscomment.length() - 12);
	} else if (thiscomment.left(8).upper() == "VERSION=") {
	    version = thiscomment.right(thiscomment.length() - 8);
	} else if (thiscomment.left(10).upper() == "PERFORMER=") {
	    performer = thiscomment.right(thiscomment.length() - 10);
	} else if (thiscomment.left(10).upper() == "COPYRIGHT=") {
	    copyright = thiscomment.right(thiscomment.length() - 10);
	} else if (thiscomment.left(8).upper() == "LICENSE=") {
	    license = thiscomment.right(thiscomment.length() - 8);
	} else if (thiscomment.left(13).upper() == "ORGANIZATION=") {
	    organization = thiscomment.right(thiscomment.length() - 13);
	} else if (thiscomment.left(12).upper() == "DESCRIPTION=") {
	    description = thiscomment.right(thiscomment.length() - 12);
	} else if (thiscomment.left(6).upper() == "GENRE=") {
	    genre = thiscomment.right(thiscomment.length() - 6);
	} else if (thiscomment.left(5).upper() == "DATE=") {
	    date = thiscomment.right(thiscomment.length() - 5);
	} else if (thiscomment.left(9).upper() == "LOCATION=") {
	    location = thiscomment.right(thiscomment.length() - 9);
	} else if (thiscomment.left(8).upper() == "CONTACT=") {
	    contact = thiscomment.right(thiscomment.length() - 8);
	} else if (thiscomment.left(5).upper() == "ISRC=") {
	    isrc =thiscomment.right(thiscomment.length() - 5);
	} else {
	    /* Non-Standard header, see for an overview:
	    http://www.xiph.org/ogg/vorbis/doc/v-comment.html */
	    DomoGlobals::logger->info("MetaInfo::getOggInfo()",
				      "Non standard comment entry found: '" + thiscomment + "' in file '" 
				   + filename + "'");
	}
    }
    
    //get the file's info
    vorbis_info *vi = ov_info(&vf, -1);
    int encVersion = vi->version;
    int channels = vi->channels;
    int sampleRate = vi->rate;
    int averageBitRate = vi->bitrate_nominal / 1000; //(in Kbps)
    
    //get the length of the song
    int songlength = (int)ov_time_total(&vf, -1);
   
    if (songlength <= 0) songlength = 0;
    if (encVersion <= 0) encVersion = 0;
    if (channels <= 0) channels = 0;
    if (sampleRate <= 0) sampleRate = 0;
    if (averageBitRate <= 0) averageBitRate = 0;
    
   //if the info is invalid, return without doing anything
   if (channels <= 0 || sampleRate <= 0) {
        DomoGlobals::logger->info("MetaInfo::getOggInfo()", 
	                                         "Channel or sampleRate not set properly for file '" + 
			         filename + "'");
        channels = 0;
        sampleRate = 0;
    }
    
    if (getArtistAndTitle) {
       if (!empty(artist)) t->setArtist(artist);
       if (!empty(title)) t->setTitle(title);
    }
    
    if (!empty(album)) t->setAlbum(album);
    if (trackNo > 0) t->setTrackNo(trackNo);
    if (!empty(description)) t->setComment(description);
    if (!empty(genre)) t->setGenre(genre);
    if (songlength > 0) t->setLength(songlength);
    
    if (sampleRate > 0) t->setSamplingRate(sampleRate);
    if (averageBitRate > 0) t->setBitrate(averageBitRate);
    //we're done reading, now clean up
    //ov_clear closes stream AND FILE, no close(file) necessary
    ov_clear(&vf);
}
//-----------------------------------------------------------------------------------------------------------
void MetaInfo::getWmaInfo(Track* t, const bool getArtistAndTitle) {
    assert(t != 0);
    //to get rid of compiler warnings
    if (getArtistAndTitle) {
    }
}
//-----------------------------------------------------------------------------------------------------------
void MetaInfo::getWavInfo(Track* t, const bool getArtistAndTitle) {
    assert(t != 0);
    //to get rid of compiler warnings
    if (getArtistAndTitle) {
    }
}
//-----------------------------------------------------------------------------------------------------------
