/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/

#include "assert.h"
#include "pgsqldbase.h"
#include "dbaseexception.h"
#include "domoglobals.h"
#include <iostream>
using std::cout;
using DbaseSpace::Dbase;
using DbaseSpace::PgSqlDbase;
//-----------------------------------------------------------------------------------------------------------
PgSqlDbase::PgSqlDbase() : Dbase() {
    setDriver("QPSQL7");
}
//-----------------------------------------------------------------------------------------------------------
PgSqlDbase::~PgSqlDbase() {
}
//-----------------------------------------------------------------------------------------------------------
void PgSqlDbase::createTmpTrackTable(const QString name) {
    
    QString q = ("CREATE TEMPORARY TABLE " + name + " LIKE Track;");
    try {
	Dbase::query(q);
    } catch(DbaseException* e) {
	throw (new DbaseException("PgSqlDbase::createTmpTrackTable",
				  "Create temp track table failed, query: '" + q + "'"));
    }
} 
//-----------------------------------------------------------------------------------------------------------
QString PgSqlDbase::getMasterTable() const {
    //Does pqsql create a standard admin table like mysql?
    return "";
}
//-----------------------------------------------------------------------------------------------------------
QString PgSqlDbase::getVendor() const {
    return "postgresql";
}
//-----------------------------------------------------------------------------------------------------------
//Incomplete
void PgSqlDbase::createNewAccount(const QString rootUser,
				  const QString rootPass,
				  const QString user,
				  const QString pass,
				  const QString host,
				  const QString database, //the database the user will create, may be used for permission granting
				  const QString connectDb) {
    
    
        DomoGlobals::logger->debug("PgSqlDbase::createNewAccount",
			       QString("Attempting to create a new database account with the following parameters: ")
			       + "\n   Root User: " + rootUser
			       + "\n   New User: " + user
			       + "\n   Host: " + host
			       + "\n   Database: " + database + " (to grant permissions to (not used yet))"
			       + "\n   Database to connect to: " + connectDb + "");
    
      QSqlDatabase* dbase = QSqlDatabase::addDatabase( getDriver(), "rootConnection");
      if (!dbase) throw (new DbaseException("PgSqlDbase::createNewAccount",
						 "Failed to connect to database driver '" 
						 + getDriver() + "'"));      
      
      //Qt cannot connect to the database server, it must connect to
      //an actual database, handy if we know there is always a standard
      //database to connect to.  If postgresql does not have such
      //a standard dbase this needs to be an existing database, owned by a user
      //who is allowed to create a new user

      QString conDb = "";
      if (connectDb.isEmpty()) {
        conDb = this->getMasterTable();
      } else {
	conDb = connectDb;
      }

      dbase->setDatabaseName( conDb );
      dbase->setUserName( rootUser );
      dbase->setPassword( rootPass );
      dbase->setHostName( host );
        
    if ( ! dbase->open() ) {
	QSqlDatabase::removeDatabase("rootConnection");
	throw (new DbaseException("PgSqlDbase::createNewAccount",
			      "Failed to open database '" + host + "@" + conDb 
			      + "' as user '" + rootUser + "' using driver '" + getDriver() + "'"));
    } else {
	QString s = "CREATE USER " + user + " WITH PASSWORD '" + pass + "' CREATEDB;";
	QSqlQuery query(s, dbase);
	
	if ( !query.isActive() ) {
	    QSqlDatabase::removeDatabase("rootConnection");
	    throw (new DbaseException("PgSqlDbase::createNewAccount",
				  "Failed to create new user '" + user + "' on '" + host + 
				  + "' as user '" + rootUser + "' using driver '" + getDriver() + "'"));            
	}
		    
	QSqlDatabase::removeDatabase("rootConnection");
    }
    
}
//-----------------------------------------------------------------------------------------------------------
int PgSqlDbase::lastInsertID(const QString table) {
    //simplistic implementation, got tired of getting lost in rubbish postgresql docs
    return queryNum("SELECT MAX(id) FROM " + table + ";");
}
//-----------------------------------------------------------------------------------------------------------
void PgSqlDbase::createDbase(const QString name){
    query("CREATE DATABASE " + name + ";");
}
//-----------------------------------------------------------------------------------------------------------
//this is the mysql version, still need to change it
QString PgSqlDbase::buildRegexpWhereClause(const QString field, QStringList& slist) {
    if (slist.empty()) return "(1<2)"; //return a where clause that is always true;
    
    QString query= "((" + field + " REGEXP ";
    QString tmp = "";
    uint size = slist.size();
    uint i = 1;
    for ( QStringList::Iterator it = slist.begin(); it != slist.end(); ++it ) {
	tmp = *it;
	if ( i < size ) {
	    query += "\"[[:<:]]" + Dbase::cleanSQL(tmp) + "[[:>:]]\") OR (" + field + " REGEXP ";
	} else {
	     query += "\"[[:<:]]" + Dbase::cleanSQL(tmp) + "[[:>:]]\"))";
	}
	++i;
    }
    //cout<<"where clause is: " << query << "\n";
    return query;
}
//-----------------------------------------------------------------------------------------------------------
void PgSqlDbase::createTables() {
    query("CREATE TABLE Wishlist ("
	  "song VARCHAR(255) NOT NULL,"
	  "PRIMARY KEY (song)"
	  ");");
    
    query("CREATE TABLE CD ("
	  "id SERIAL NOT NULL UNIQUE,"
	  "title VARCHAR(100) NOT NULL UNIQUE,"
	  "type ENUM('MP3_CD','AUDIO_CD','DATA_CD','MOVIE_CD') NOT NULL,"
	  "PRIMARY KEY (id,title));");
    
    
    query("CREATE TABLE Album ("
	  "title VARCHAR(100) NOT NULL,"
	  "cdno INT REFERENCES CD(id),"
	  "id SERIAL NOT NULL UNIQUE,"
	  "format ENUM('DIG_AUDIO','CDA') NOT NULL,"
	  "tracks INT NOT NULL CHECK (tracks > 0),"
	  "PRIMARY KEY (id,title));");
    
    query("CREATE TABLE File ("
	  "id SERIAL NOT NULL,"			
	  "cdno INT REFERENCES CD(id),"
	  "name VARCHAR(255) NOT NULL,"
	  "path VARCHAR(255) NOT NULL,"
	  "descr VARCHAR(255),"
	  "size INT CHECK (size >= 0),"
	  "created VARCHAR(25),"
	  "modified VARCHAR(25),"
	  "accessed VARCHAR(25),"
	  "attributes VARCHAR(25),"
	  "PRIMARY KEY (id));");
    
    query("CREATE TABLE Track ("
	  "id SERIAL NOT NULL,"			
	  "cdno INT REFERENCES CD(id),"
	  "name VARCHAR(255) NOT NULL,"
	  "path VARCHAR(255) NOT NULL,"
	  "descr VARCHAR(255),"
	  "size INT CHECK (size >= 0),"
	  "created VARCHAR(25),"
	  "modified VARCHAR(25),"
	  "accessed VARCHAR(25),"
	  "attributes VARCHAR(25),"
	  //track attributes
	  "trackno INT CHECK (trackno >= 0),"
	  "artist VARCHAR(255) NOT NULL,"
	  "title VARCHAR(255) NOT NULL,"
	  "album VARCHAR(100),"
	  "length INT CHECK (length >= 0),"
	  "bitrate INT CHECK (bitrate >= 0),"
	  "samplingrate INT CHECK (samplingrate >= 0),"
	  "year INT CHECK ((year >= 1900) AND (year <= 2050)),"
	  "mood VARCHAR(50),"
	  "genre VARCHAR(50),"
	  "comment VARCHAR(255),"
	  "score INT CHECK ((score >= 0)AND(score <= 10)),"
	  "PRIMARY KEY (id));");
    
    query("CREATE TABLE Mp3CD ("
	  "id INT NOT NULL REFERENCES CD(id),"
	  "trackcount INT CHECK (trackcount > 0),"
	  "title VARCHAR(100) REFERENCES CD(title),"
	  "genre VARCHAR(50) DEFAULT 'Various',"
	  "comment VARCHAR(255),"
	  "PRIMARY KEY (id));");
    
    query("CREATE TABLE AudioCD ("
	  "id INT NOT NULL REFERENCES CD(id),"
	  "trackcount INT CHECK (trackcount > 0),"
	  "title VARCHAR(100) REFERENCES CD(title),"
	  "artist VARCHAR(100) DEFAULT 'Various',"
	  "year INT CHECK ((year >= 1900)AND(year <= 2050)),"
	  "genre VARCHAR(50) DEFAULT 'Various',"
	  "comment VARCHAR(255),"
	  "PRIMARY KEY (id));");				
    
    
    query("CREATE TABLE MovieCD ("
	  "id INT NOT NULL REFERENCES CD(id),"
	  "filecount INT CHECK (filecount >= 0),"
	  "title VARCHAR(100) REFERENCES CD(title),"
	  "genre VARCHAR(50) DEFAULT 'Various',"
	  "comment VARCHAR(255),"
	  "PRIMARY KEY (id));");
    
    query("CREATE TABLE DataCD ("
	  "id INT NOT NULL REFERENCES CD(id),"
	  "filecount INT CHECK (filecount >= 0),"
	  "title VARCHAR(100) REFERENCES CD(title),"
	  "volumelabel VARCHAR(50),"
	  "comment VARCHAR(255),"
	  "PRIMARY KEY (id));");
    
    query("CREATE TABLE Collection ("
	  "owner VARCHAR(50) NOT NULL,"
	  "descr VARCHAR(50),"
	  "title VARCHAR(50) NOT NULL DEFAULT 'myCollection',"
	  "mp3CDcount INT CHECK (mp3CDcount >= 0),"
	  "audioCDcount INT CHECK (audioCDcount >= 0),"
	  "movieCDcount INT CHECK (movieCDcount >= 0),"
	  "dataCDcount INT CHECK (dataCDcount >= 0),"
	  "comment VARCHAR(255),"
	  "PRIMARY KEY (title));");
    
    query("CREATE TABLE Contains ("
	  "title VARCHAR(50) NOT NULL REFERENCES Collection(title),"
	  "id INT NOT NULL REFERENCES CD(id),"
	"PRIMARY KEY (id,title));");
}
//-----------------------------------------------------------------------------------------------------------
