/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/
#ifndef INC_IOEXCEPTION_H
#include "ioexception.h"
#define INC_IOEXCEPTION_H
#endif

#ifndef INC_LOG_H
#include "log.h"
#define INC_LOG_H
#endif

#ifndef INC_STRINGMANIP_H
#include "stringmanip.h"
#define INC_STRINGMANIP_H
#endif

#ifndef INC_RCMAN_H
#include "rcman.h"
#define INC_RCMAN_H
#endif

#ifndef INC_WRITERTXT_H
#include "writertxt.h"
#define INC_WRITERTXT_H
#endif

#include "domoglobals.h"
#include "assert.h"

#include <iostream>
using std::cout;

using WriterSpace::WriterTxt;
using ReaderSpace::ReaderTxt;
using RcManSpace::RcMan;
//-----------------------------------------------------------------------------------------------------------
RcMan::RcMan(const Settings* s){
    fExtInfoFile = DomoGlobals::DOMO_DIR + s->get(Settings::fExtInfoFile);
    fGenreListFile = DomoGlobals::DOMO_DIR + s->get(Settings::fGenreListFile);
    fArtistSafeListFile = DomoGlobals::DOMO_DIR + s->get(Settings::fArtistSafeListFile);
    fKeywordFile = DomoGlobals::DOMO_DIR + s->get(Settings::fKeywordsFile);
    fHyphenedArtistFile = DomoGlobals::DOMO_DIR + s->get(Settings::fHyphenedArtistsFile);
}
//-----------------------------------------------------------------------------------------------------------
void RcMan::parseAll() {
    DomoGlobals::logger->info("RcMan::parseAll", "Starting resource read...");
    QString tmp = "";
    
    if (fExtInfoFile != "") {
	readExtensionInfo();
	DomoGlobals::logger->info("RcMan::parseAll", tmp.setNum(fExtInfoList.size()) + " entries read from '" + fExtInfoFile + "'");
    }
    tmp = "";
    if (fGenreListFile != "") {
	read(fGenreList,fGenreListFile);
	DomoGlobals::logger->info("RcMan::parseAll", tmp.setNum(fGenreList.size()) + " genres read from '" + fGenreListFile + "'");
    }
    
    tmp = "";
    if (fArtistSafeListFile != "") {
	read(fArtistSafeList,fArtistSafeListFile);
	DomoGlobals::logger->info("RcMan::parseAll",  tmp.setNum(fArtistSafeList.size()) + " entries read from '" + fArtistSafeListFile + "'");
    }
    
    tmp = "";
    if (fKeywordFile != "") {
	read(fKeywordList,fKeywordFile);
	DomoGlobals::logger->info("RcMan::parseAll",  tmp.setNum(fKeywordList.size()) + " entries read from '" + fKeywordFile + "'");
    }
    
    tmp = "";
    if (fHyphenedArtistFile != "") {
	read(fHyphenedArtistList,fHyphenedArtistFile);
	DomoGlobals::logger->info("RcMan::parseAll",  tmp.setNum(fHyphenedArtistList.size()) + " entries read from '" + fHyphenedArtistFile + "'");
    }
}
//-----------------------------------------------------------------------------------------------------------
RcMan::~RcMan(){
}
//-----------------------------------------------------------------------------------------------------------
bool RcMan::isSafeArtist(const QString artist) const {
    bool found = false;
    QString tmp = artist.upper();
    for ( QStringList::ConstIterator it = fArtistSafeList.begin(); (it != fArtistSafeList.end()) && !found; ++it ) {
	if( (*it).upper() == tmp ) found = true;
    }
    return found;
}
//-----------------------------------------------------------------------------------------------------------
bool RcMan::startsWithSafeArtist(const QString s1) const {
    QString tmp = "";
    QString s2 = (s1.upper()).stripWhiteSpace();
    
    for ( QStringList::ConstIterator it = fArtistSafeList.begin(); it != fArtistSafeList.end(); ++it ) {
	tmp = (*it).upper();
	//cout<<"checking: " << tmp << "\n";
	if (s2.startsWith(tmp)) {
	    //cout<< s2 << " starts with " << tmp << "\n";
	    return true;
	}
    }
   //cout<< s1 << " can be stripped safely\n";
    return false;
}
//-----------------------------------------------------------------------------------------------------------
QString RcMan::resolveGenre(const uint i) const {
    QString genre = "";
    if(i >= fGenreList.size()) {
	genre = "";
    } else {
	genre = fGenreList[i];
    }
    
    return genre;
}
//-----------------------------------------------------------------------------------------------------------
void RcMan::read(QStringList& slist,const QString s) const {
	ReaderTxt r(s);
	r.read(slist);
}
//-----------------------------------------------------------------------------------------------------------
void RcMan::readExtensionInfo() {
    ReaderTxt r(fExtInfoFile);
    r.readExtInfo(fExtInfoList);
}
//-----------------------------------------------------------------------------------------------------------
QString RcMan::getExtDesc(const QString ext) {
    return  fExtInfoList[ext.upper()];
}
//-----------------------------------------------------------------------------------------------------------
QStringList* RcMan::getGenreListRef(){
    return (&fGenreList);
}
 //-----------------------------------------------------------------------------------------------------------
bool RcMan::isKeyword(const QString s) const {
    //track numbers (eg: 03) and strings with length < 2 are not considered keywords
    bool b = ( (s.length() == 2 && isdigit(s[0]) && isdigit(s[1])) || (s.length() < 2) );
    //cout <<"keyword list is: "<< fKeywordList.join(" : ") << "\n";
    
    if (b) {
	return false;
    } else { 
	return ! fKeywordList.contains(s.upper());
    }
}
//-----------------------------------------------------------------------------------------------------------
bool RcMan::containsHyphenedArtist(const QString s) const{
    QString tmp = "";
    for ( QStringList::ConstIterator it = fHyphenedArtistList.begin(); it != fHyphenedArtistList.end(); ++it ) {
	tmp = (*it);
	//cout<<"checking: " << tmp << "\n";
	if (s.contains(tmp,false)) {
	    //cout<< s2 << " contains " << tmp << "\n";
	    return true;
	}
    }
    
  return false;
}
//-----------------------------------------------------------------------------------------------------------
void RcMan::addHyphenedArtist(const QString s) {
    if ((s != "")&&(dashCount(s) == 1)) fHyphenedArtistList.append(s);
}
//-----------------------------------------------------------------------------------------------------------
void RcMan::flushHyphenedArtists() const {
    WriterTxt writer(fHyphenedArtistFile);
    
    QString s = "#######################################################\n";
	s +=  "# DOMO v" + DomoGlobals::DOMO_VERSION + " Resource file                             #\n";
	s +=  "#                                                     #\n";
	s +=  "# Domo is a GNU GPL licensed product by GoriTech      #\n";
	s +=  "#                                                     #\n";
	s +=  "# To ease the splitting of a trackname into its       #\n";
	s +=  "# respective Artist/Title fields this file contains   #\n";
	s +=  "# a list of artists whose name contains ONE hyphen    #\n";
	s +=  "#                                                     #\n";
	s +=  "#######################################################\n";
    
    writer.writeHeader(s);
    writer.writeLn();
    
    QString tmp = "";
    for ( QStringList::ConstIterator it = fHyphenedArtistList.begin(); it != fHyphenedArtistList.end(); ++it ) {
	writer.writeLn( (*it) );
    }
    
}
//-----------------------------------------------------------------------------------------------------------
