/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/
#ifndef INC_IOEXCEPTION_H
#include "ioexception.h"
#define INC_IOEXCEPTION_H
#endif

#ifndef INC_TEXTISTREAM_H
#include "textistream.h"
#define INC_TEXTISTREAM_H
#endif

#include "qfile.h"
#include "assert.h"
#include <iostream>

using std::cout;

//-----------------------------------------------------------------------------------------------------------
TextIStream::TextIStream(const QString fname) : QTextStream() {
    //cout<<"filename is: " << fname << "\n";
    QFile* f = new QFile(fname);
    assert(f != 0);
    bool ok = f->open(IO_ReadOnly|IO_Translate);
    if (!ok) {
	throw (new IoException("TextIStream::TextIStream","File '" + fname + "' could not be opened"));
    }
   
    setDevice(f);        
    ok = device()->isReadable();
    if (!ok) {
	throw (new IoException("TextIStream::TextIStream","File '" + fname + "' could not be opened for reading"));
    }
    fName = fname;
    fLine = 1;
}
//-----------------------------------------------------------------------------------------------------------
TextIStream::~TextIStream() {
}
//-----------------------------------------------------------------------------------------------------------
bool TextIStream::isOpen() const {
    if (device() == 0) return false; //throw (new IoException("TextIStream::isOpen","device() is 0, file is '" + fName + "'"));
    return device()->isOpen();
}
//-----------------------------------------------------------------------------------------------------------
bool TextIStream::atEnd() const {
    return QTextStream::atEnd();
}
//-----------------------------------------------------------------------------------------------------------
bool TextIStream::isGood() const {
    return (isOpen() && !atEnd() );
}
//-----------------------------------------------------------------------------------------------------------
bool TextIStream::isComment(const QString s) {
    return s.startsWith("#");
}
//-----------------------------------------------------------------------------------------------------------
void TextIStream::getLine(QString& s, const bool ignoreComments, const bool ignoreEmptyLines) {
	++fLine;
    QString t ="";
	t = readLine();  //returns null when eof
	t = t.stripWhiteSpace ();
	
	if (ignoreComments) {
	    if ( isComment( t ) && isGood() ) {
		getLine(t,ignoreComments,ignoreEmptyLines);
	    }
	}
	
	if (ignoreEmptyLines) {
	    if ( (t.isEmpty()) && isGood() ) {
		getLine(t,ignoreComments,ignoreEmptyLines);   
	    }
	}
	
	t = t.stripWhiteSpace();
	
	if (ignoreComments) {
	    if ( isComment( t ) ) t = "";
	}
	s = t;
	//cout<<"Line read: " << s << "\n";
}
//-----------------------------------------------------------------------------------------------------------
/* void TextIStream::get(char& ch) {
	int i = -1;
	i = device()->getch();  //returns -1 when eof

	if (i == -1) {
		ch = 0;
	} else {
		ch = i;
	}
} */
//-----------------------------------------------------------------------------------------------------------
int TextIStream::line() {
    return fLine;
}
//-----------------------------------------------------------------------------------------------------------
