/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/

#ifndef INC_TRACK_H
#include "track.h"
#define INC_TRACK_H
#endif

#ifndef INC_STRINGMANIP_H
#include "stringmanip.h"
#define INC_STRINGMANIP_H
#endif

#ifndef INC_DOMOGLOBALS_H
#include "domoglobals.h"
#define INC_DOMOGLOBALS_H
#endif

#ifndef INC_IOEXCEPTION_H
#include "ioexception.h"
#define INC_IOEXCEPTION_H
#endif

using std::endl;
using std::vector;
using std::ostream;
using TrackSpace::Track;
using FileSpace::File;

//-----------------------------------------------------------------------------------------------------------
Track::Track() : File(){
    fTitle = "";
    fArtist = "";
    fAlbum = "";
    fComment = "";
    fScore = 0;
    fMood = "";
    fTrackNo = 0;
    fLength = 0;
    fBitrate = 0;
    fSamplingRate = 0;
    fYear = 0;
    fGenre = "";
}
//-----------------------------------------------------------------------------------------------------------
Track::Track(const QString a, const QString t, const int n) : File(){
    fArtist = a;
    fTitle = t;
    fTrackNo = n;
    fAlbum = "";
    fComment = "";
    fScore = 0;
    fMood = "";
    fLength = 0;
    fBitrate = 0;
    fSamplingRate = 0;
    fYear = 0;
    fGenre = "";
}
//-----------------------------------------------------------------------------------------------------------
Track::Track(const QString a, const QString t, const QString comment, const int num) : File(){
    fArtist = a;
    fTitle = t;
    fTrackNo = num;
    fAlbum = "";
    fComment = comment;
    fScore = 0;
    fMood = "";
    fLength = 0;
    fBitrate = 0;
    fSamplingRate = 0;
    fYear = 0;
    fGenre = "";
}
//-----------------------------------------------------------------------------------------------------------
Track::Track(const Track& t) : File(){
    setName(t.getName());
    setDescription(t.getDescription());
    setSize(t.getSize());
    setCreated(t.getCreated());
    setModified(t.getModified());
    setAccessed(t.getAccessed());
    setPath(t.getPath());
    setAttributes(t.getAttributes());
    setCDno(t.getCDno());
    setID(t.getID());
    
    fArtist = t.getArtist();
    fTitle = t.getTitle();
    fAlbum = t.getAlbum();
    fComment = t.getComment();
    fScore = t.getScore();
    fMood = t.getMood();
    fTrackNo = t.getTrackNo();
    fLength = t.getLength();
    fBitrate = t.getBitrate();
    fSamplingRate = t.getSamplingRate();
    fYear = t.getYear();
    fGenre = t.getGenre();
}
//-----------------------------------------------------------------------------------------------------------
Track::~Track(){
}
//-----------------------------------------------------------------------------------------------------------
QString Track::getArtist() const{
	return fArtist;
}
//-----------------------------------------------------------------------------------------------------------
void Track::setArtist(const QString a){
	fArtist = a;
}
//-----------------------------------------------------------------------------------------------------------
QString Track::getTitle() const{
	return fTitle;
}
//-----------------------------------------------------------------------------------------------------------
void Track::setTitle(const QString t){
	fTitle = t;
}
//-----------------------------------------------------------------------------------------------------------
QString Track::getAlbum() const{
	return fAlbum;
}
//-----------------------------------------------------------------------------------------------------------
void Track::setAlbum(const QString a){
	fAlbum = a;
}
//-----------------------------------------------------------------------------------------------------------
QString Track::getGenre() const{
	return fGenre;
}
//-----------------------------------------------------------------------------------------------------------
void Track::setMood(const QString g){
	fMood = g;
}
//-----------------------------------------------------------------------------------------------------------
QString Track::getMood() const{
	return fMood;
}
//-----------------------------------------------------------------------------------------------------------
void Track::setGenre(const QString g){
	fGenre = g;
}
//-----------------------------------------------------------------------------------------------------------
QString Track::getComment() const{
	return fComment;
}
//-----------------------------------------------------------------------------------------------------------
void Track::setComment(const QString c){
	fComment = c;
}
//-----------------------------------------------------------------------------------------------------------
void Track::appendComment(const QString c){
	if (fComment != "") {
		fComment += " , " + c;
	} else {
		fComment = c;
	}
}
//-----------------------------------------------------------------------------------------------------------
int Track::getLength() const{
	return fLength;
}
//-----------------------------------------------------------------------------------------------------------
void Track::setLength(const int i){
	fLength = i;
}
//-----------------------------------------------------------------------------------------------------------
void Track::setLength(const QString i){
	fLength = i.toLong();
}
//-----------------------------------------------------------------------------------------------------------
int Track::getBitrate() const{
	return fBitrate;
}
//-----------------------------------------------------------------------------------------------------------
void Track::setBitrate(const int i){
	fBitrate = i;
}
//-----------------------------------------------------------------------------------------------------------
void Track::setBitrate(const QString i){
	fBitrate = i.toInt();
}
//-----------------------------------------------------------------------------------------------------------
int Track::getSamplingRate() const{
	return fSamplingRate;
}
//-----------------------------------------------------------------------------------------------------------
void Track::setSamplingRate(const int i){
	fSamplingRate = i;
}
//-----------------------------------------------------------------------------------------------------------
void Track::setSamplingRate(const QString i){
	fSamplingRate = i.toInt();
}
//-----------------------------------------------------------------------------------------------------------

int Track::getYear() const{
	return fYear;
}
//-----------------------------------------------------------------------------------------------------------
void Track::setYear(const int i){
	fYear = i;
}
//-----------------------------------------------------------------------------------------------------------
void Track::setYear(const QString i){
	fYear = i.toInt();
}
//-----------------------------------------------------------------------------------------------------------
int Track::getScore() const{
	return fScore;
}
//-----------------------------------------------------------------------------------------------------------
void Track::setScore(const int i){
	fScore = i;
}
//-----------------------------------------------------------------------------------------------------------
void Track::setScore(const QString i){
	fScore = i.toInt();
}
//-----------------------------------------------------------------------------------------------------------
int Track::getTrackNo() const{
	return fTrackNo;
}
//-----------------------------------------------------------------------------------------------------------
void Track::setTrackNo(const int i){
	fTrackNo = i;
}
//-----------------------------------------------------------------------------------------------------------
void Track::setTrackNo(const QString i){
	fTrackNo = i.toInt();
}

//-----------------------------------------------------------------------------------------------------------
void Track::writeTxt(TextOStream* ostr) const{
    if ((ostr == 0)||(!ostr->isGood())) throw (new IoException("Track::writeTxt","Pre: Ostream is 0 or not good"));
    
     switch (ostr->getMode()) {
    case TextOStream::SHORT:
	*ostr << fArtist << " - " << fTitle;
	break;
    case TextOStream::LONG:
	if (!empty(fTrackNo))               *ostr<<"Track Number: "	<< fTrackNo	<< "\n";
	*ostr<<"Artist: "		<< fArtist		<< "\n";
	*ostr<<"Title: "		<< fTitle		<< "\n";
	if (!empty(fAlbum))	*ostr<<"Album: "		<< fAlbum		<< "\n";
	if (!empty(fLength))	*ostr<<"Length: "		<< fLength	<<  " s\n";
	if (!empty(fBitrate))		*ostr<<"Bitrate: "		<< fBitrate	<< " Kbps\n";
	if (!empty(fSamplingRate))	 *ostr<<"Sampling Rate: "	<< fSamplingRate  	<< " Hz\n";
	if (!empty(fYear))		*ostr<<"Year: "		<< fYear		<< "\n";
	if (!empty(fMood))		*ostr <<"Mood: "		<< fMood		<< "\n";
	if (!empty(fGenre))		*ostr <<"Genre: "		<< fGenre		<< "\n";
	if (!empty(fComment))	*ostr<<"Comment: "		<< fComment	<< "\n";
	if (!empty(fScore))		*ostr<<"Score: "		<< fScore		<< "\n";
	
	File::writeTxt(ostr);
    	break;
    default:
	break;
    }
    
     if ((ostr == 0)||(!ostr->isGood())) throw (new IoException("Track::writeTxt","Post: Ostream is 0 or not good"));
}
//-----------------------------------------------------------------------------------------------------------
void Track::writeHtml(TextOStream* ostr) const {
    if ((ostr == 0)||(!ostr->isGood())) throw (new IoException("Track::writeHtml","Pre: Ostream is 0 or not good"));
    
    switch (ostr->getMode()) {
    case TextOStream::SHORT:
	*ostr << fArtist << " - " << fTitle << "<br>\n";
	break;
    case TextOStream::LONG:
	 File::writeHtml(ostr);
	 *ostr << "<ul>\n";
	
	if (!empty(fTrackNo))    	*ostr << "   <li>" << "<b>No: </b>" 		<< fTrackNo 	<< "</li>\n";
	                                          *ostr << "   <li>" << "<b>Title: </b>"      		<< fTitle 		<< "</li>\n";
				*ostr << "   <li>" << "<b>Artist: </b>"     		<< fArtist        	<< "</li>\n";
	if (!empty(fAlbum)) 	*ostr << "   <li>" << "<b>Album: </b>"   		<< fAlbum 	<< "</li>\n";
	if (!empty(fYear))    	*ostr << "   <li>" << "<b>Year: </b>"  		<< fYear 	<< "</li>\n";
	if (!empty(fLength))    	*ostr << "   <li>" << "<b>Length: </b>"		 << fLength 	<< " s</li>\n";
	if (!empty(fBitrate))    	*ostr << "   <li>" << "<b>Bitrate: </b>"		 << fBitrate 	<< " Kbps</li>\n";
	if (!empty(fSamplingRate))   *ostr << "   <li>" << "<b>Sampling Rate: </b>" 	<< fSamplingRate << " Hz</li>\n";
	if (!empty(fGenre)) 	*ostr << "   <li>" << "<b>Genre: </b>"  		<< fGenre 	<< "</li>\n";
	if (!empty(fMood)) 	*ostr << "   <li>" << "<b>Mood: </b>"   		<< fMood 	<< "</li>\n";
	if (!empty(fScore))    	*ostr << "   <li>" << "<b>Score: </b>"  		<< fScore 	<< "</li>\n";
	if (!empty(fComment)) 	*ostr << "   <li>" << "<b>Comment: </b>"		<< fComment 	<< "</li>\n";
	
	*ostr << "</ul>\n";
	break;
    default:
	break;
    }
	
    if ((ostr == 0)||(!ostr->isGood())) throw (new IoException("Track::writeHtml","Post: Ostream is 0 or not good"));

}
//-----------------------------------------------------------------------------------------------------------
//M3U format -> only for Music
void Track::writeM3u(TextOStream* ostr) const {
    if ((ostr == 0)||(!ostr->isGood())) throw (new IoException("Track::M3u","Pre: Ostream is 0 or not good"));
    
    *ostr << "#EXTINF:";
    int len = fLength;
    if (len <= 0) {
	*ostr << -1;
    } else {
	*ostr << len;
    }

    *ostr << ',' << fArtist << " - " << fTitle;
     if (fComment != "") *ostr << " (" << fComment << ")";
    *ostr << "\n";
    
    if (getPath() != "") {
	*ostr << getPath() << DomoGlobals::DIR_SEP;
    }
    *ostr << getName() << '\n';    
         
    if ((ostr == 0)||(!ostr->isGood())) throw (new IoException("Track::M3u","Post: Ostream is 0 or not good"));
}
//-----------------------------------------------------------------------------------------------------------
void Track::writePls(TextOStream* ostr, const uint number) const {
    if ((ostr == 0)||(!ostr->isGood())) throw (new IoException("Track::writePls","Pre: Ostream is 0 or not good"));
    
    uint num = fTrackNo;
    if (number > 0) num = number;
    
    *ostr << "File" << num << "=";
    if (getPath() != "") {
	*ostr << getPath() << DomoGlobals::DIR_SEP;
    }
    *ostr << getName() << "\n";
    
    *ostr << "Title" << num << "=";
    *ostr << fArtist << " - " << fTitle;
    if (fComment != "") *ostr << " (" << fComment << ")";
    *ostr << "\n";
    
    *ostr << "Length" << num << "=" << fLength<<"\n\n";
    
    if ((ostr == 0)||(!ostr->isGood())) throw (new IoException("Track::writePls","Post: Ostream is 0 or not good"));
}
//-----------------------------------------------------------------------------------------------------------
//B4s format -> only for Music
void Track::writeB4s(TextOStream* ostr) const {
    if ((ostr == 0)||(!ostr->isGood())) throw (new IoException("Track::writeB4s","Pre: Ostream is 0 or not good"));
    
    /* Example:
    <entry Playstring="file:D:\done\music\_SCOOTER____THE_AGE_OF_LOVE.MP3">
      <Name>Scooter - The Age Of Love</Name>
      <Length>228624</Length>
     </entry>
    */
    
    QString path = "";
    if (getPath() != "") path = getPath() + DomoGlobals::DIR_SEP;
    path += getName();    
    
    *ostr << "\t<entry Playstring=\"file:" << cleanXml( path ) << "\">\n";
    *ostr << "\t\t<Name>" << cleanXml( getArtist() ) << " - " << cleanXml( getTitle() );
    if (!empty(getComment())) *ostr << " (" << cleanXml( getComment() ) << ")";
  
    *ostr << "</Name>\n";
    *ostr << "\t\t<Length>" + toString(getLength()*1000) + "</Length>\n"; //length must be in milliseconds
    *ostr << "\t</entry>\n\n";
    
    if ((ostr == 0)||(!ostr->isGood())) throw (new IoException("Track::writeB4s","Post: Ostream is 0 or not good"));
}
//-----------------------------------------------------------------------------------------------------------
void Track::writeCsv(TextOStream* ostr,const QStringList& fields) const {
    if ((ostr == 0)||(!ostr->isGood())) throw (new IoException("Track::writeCsv","Pre: Ostream is 0 or not good"));
    
    File::writeCsv(ostr,fields);
    
    if (fields.contains("TrackNo")) *ostr << fTrackNo    << ";";
    if (fields.contains("Artist")) *ostr << fArtist              << ";";
    if (fields.contains("Title")) *ostr << fTitle        << ";";
    if (fields.contains("Album")) *ostr << fAlbum   << ";";
    if (fields.contains("Length")) *ostr   << fLength      << ";";
    if (fields.contains("Bitrate")) *ostr   << fBitrate     << ";";
    if (fields.contains("Samplingrate")) *ostr << fSamplingRate   << ";";
    if (fields.contains("Year")) *ostr << fYear         << ";";
    if (fields.contains("Mood")) *ostr << fMood    << ";";
    if (fields.contains("Genre")) *ostr << fGenre    << ";";
    if (fields.contains("Comment")) {
    	QString tmp = fComment;
	*ostr << tmp.replace(QChar(';'),QChar(' '))    << ";";
    }
    if (fields.contains("Score")) *ostr << fScore    << ";";
    
    if ((ostr == 0)||(!ostr->isGood())) throw (new IoException("Track::writeCsv","Post: Ostream is 0 or not good"));
}
//-----------------------------------------------------------------------------------------------------------
bool Track::fishyQuality() const {
    if ((fLength != 0)&&( (fLength < 60) || (fLength >= 900) ) ) return true;
    if ((fBitrate != 0)&&(fBitrate < 128)) return true;
    return false;
}
//-----------------------------------------------------------------------------------------------------------
void Track::deleteTracks(Tracklist& tlist) {
    TrackIterator it = tlist.begin();
    while (it != tlist.end()){
	delete (*it);
	++it;
    }
    tlist.clear();
}
//-----------------------------------------------------------------------------------------------------------
ostream& operator<<(ostream & out, Track& t){
	out<<"Track Number: "	<< t.getTrackNo()		<< endl;
	out<<"Artist: "		<< t.getArtist()		<< endl;
	out<<"Title: "		<< t.getTitle()		<< endl;
	out<<"Album: "		<< t.getAlbum()		<< endl;
	out<<"Length: "		<< t.getLength()		<< endl;
	out<<"Bitrate: "		<< t.getBitrate()		<< endl;
	out<<"Sampling Rate: "	<< t.getSamplingRate()  	<< endl;
	out<<"Year: "		<< t.getYear()		<< endl;
	out<<"Mood: "		<< t.getMood()		<< endl;
	out<<"Genre: "		<< t.getGenre()		<< endl;
	out<<"Comment: "	<< t.getComment()	<< endl;
	out<<"Score: "		<< t.getScore()		<< endl;
	
	return out;
}
//-----------------------------------------------------------------------------------------------------------
Track* TrackSpace::makeTrack(const QString& s2) {
	QString s = stripExtension(s2);
	QString artist = "";
	QString title = "";
	QString comment = "";
	
	splitTrackname(s,artist,title,comment, 1);
		
	Track* t = new Track(artist, title);
	t->setComment(comment);		
	t->setName(s2);
	return t;
}
//-----------------------------------------------------------------------------------------------------------
bool TrackSpace::equalLength(const Track& t1, const Track& t2, const int tolerance) { //tolerance in sec
	return ((t1.getLength() > t2.getLength() - tolerance) && (t1.getLength() < t2.getLength() + tolerance)) ;
}
//-----------------------------------------------------------------------------------------------------------
Track* TrackSpace::castToTrack(File* f) {
	Track* t = new Track();

	try {
	  t = dynamic_cast<Track*>(f);
	} catch (std::bad_cast) {
	}
	 
     return t; 
}
//-----------------------------------------------------------------------------------------------------------
